import os
from typing import List
import click
from pathlib import Path
from rich.console import Console

from pylocc.processor import Processor, ProcessorConfiguration, ProcessorConfigurationFactory
from pylocc.reporter import report_aggregate, report_by_file


def load_language_config() -> List[ProcessorConfiguration]:
    """Load language configurations from the packaged JSON file."""
    import json
    from importlib import resources

    try:
        # Use importlib.resources to access the file
        with resources.files('pylocc').joinpath('language.json').open('r', encoding='utf-8') as f:
            config_data = json.load(f)
        return ProcessorConfiguration.load_from_dict(config_data)
    except (FileNotFoundError, json.JSONDecodeError) as e:
        # Handle potential errors during file loading or parsing
        click.echo(f"Error loading language configuration: {e}", err=True)
        raise click.Abort()


def get_all_file_paths_pathlib(folder: str, supported_extensions: List[str] = []) -> List[str]:
    """
    Returns a list of all file paths using pathlib (more modern approach).

    Args:
        folder (str): Path to the root folder to search

    Returns:
        List[str]: List of absolute file paths as strings
    """
    folder_path = Path(folder)

    if not folder_path.exists():
        raise FileNotFoundError(f"The path '{folder_path}' does not exist")
    if not folder_path.is_dir():
        raise NotADirectoryError(
            f"The path '{folder_path}' is not a directory")

    # Use rglob to recursively find all files
    file_paths = [str(file.resolve())
                  for file in folder_path.rglob('*') if file.is_file()]

    # Filter by supported extensions if provided
    if supported_extensions:
        extensions_set = set(supported_extensions)
        file_paths = [file for file in file_paths if Path(
            file).suffix[1:] in extensions_set]

    return file_paths


@click.command()
@click.argument('file', type=click.Path(exists=True, dir_okay=True, readable=True))
@click.option('--by-file', is_flag=True,
              help='Generate report by file.')
@click.option('--output', type=click.Path(exists=False, dir_okay=False, readable=True, writable=True),
              help='Stores the output report in csv format to the given path')
def pylocc(file, by_file, output):
    """Run pylocc on the specified file or directory."""
    configs = load_language_config()
    supported_extensions = [
        ext for config in configs for ext in config.file_extensions]

    configuration_factory = ProcessorConfigurationFactory(configs)

    if os.path.isdir(file):
        files = get_all_file_paths_pathlib(
            file, supported_extensions=supported_extensions)
    else:
        files = [file]

    processor = Processor()
    per_file_reports = {}
    for f in files:
        try:
            with open(f, 'r', encoding='utf-8', errors='ignore') as f_handle:
                content = f_handle.readlines()
            file_extension = os.path.splitext(f)[1][1:]
            file_configuration = configuration_factory.get_configuration(
                file_extension)
            if file_configuration:
                report = processor.process(
                    content, file_configuration=file_configuration)
                per_file_reports[f] = report
            else:
                click.echo(
                    f"No configuration found for file type '{file_extension}' in file {f}. Skipping...")
                continue
        except Exception as e:
            click.echo(f"Error processing file {f}: {e} Skipping...")
            continue
    if per_file_reports:
        console = Console()
        report_table = None
        if by_file:
            report_table = report_by_file(per_file_reports)
        else:
            report_table = report_aggregate(per_file_reports)
        console.print(report_table)


if __name__ == '__main__':
    pylocc()



from typing import Dict
from pylocc.processor import Report
from rich.table import Table

# Language,Provider,Filename,Lines,Code,Comments,Blanks,Complexity,Bytes,ULOC
file_type_header = "Language"
file_path_header = "Provider"
num_file_header = "Files"
total_line_header = "Lines"
code_line_header = "Code"
comment_line_header = "Comments"
blank_line_header = "Blanks"


def report_by_file(processed: Dict[str, Report]) -> Table:
    """Given the report by file, prepares a table contaning the results.
    Also generates an overall report aggregating all the informations"""
    report = Table(show_header=True, header_style="bold magenta")
    report.add_column(file_path_header, style="dim")
    report.add_column(total_line_header, justify="right")
    report.add_column(code_line_header, justify="right")
    report.add_column(comment_line_header, justify="right")
    report.add_column(blank_line_header, justify="right")

    for file_path, report_data in processed.items():
        report.add_row(
            file_path,
            f"{report_data.total:,}",
            f"{report_data.code:,}",
            f"{report_data.comments:,}",
            f"{report_data.blanks:,}",
        )
    return report


def report_aggregate(processed: Dict[str, Report]) -> Table:
    """Given the report by file, prepares a table containing the overall results."""
    # Initialize accumulators for the overall aggregated numbers
    total_lines = 0
    code_lines = 0
    blank_lines = 0
    comment_lines = 0
    total_files = 0

    # Initialize the aggregators for the per file type report
    aggregated_report = {}
    files_per_type = {}
    for report_data in processed.values():
        if report_data.file_type not in aggregated_report:
            # If there is no accumulator yet, initialize it
            aggregated_report[report_data.file_type] = Report(
                file_type=report_data.file_type)
            # as well as it's corresponding file counter
            files_per_type[report_data.file_type] = 0

        # Increment the accumulators for the code statistics
        aggregated_report[report_data.file_type].increment_code(
            report_data.code)
        aggregated_report[report_data.file_type].increment_comments(
            report_data.comments)
        aggregated_report[report_data.file_type].increment_blanks(
            report_data.blanks)
        # Increment the total files counter
        files_per_type[report_data.file_type] += 1

    report = Table(show_header=True, header_style="bold magenta")
    report.add_column(file_type_header, style="dim")
    report.add_column(num_file_header, justify="right")
    report.add_column(total_line_header, justify="right")
    report.add_column(code_line_header, justify="right")
    report.add_column(comment_line_header, justify="right")
    report.add_column(blank_line_header, justify="right")

    for file_type, report_data in aggregated_report.items():
        report.add_row(
            file_type,
            f"{files_per_type[file_type]:,}",
            f"{report_data.total:,}",
            f"{report_data.code:,}",
            f"{report_data.comments:,}",
            f"{report_data.blanks:,}",
        )
        total_files += files_per_type[report_data.file_type]
        total_lines += report_data.total
        code_lines += report_data.code
        comment_lines += report_data.comments
        blank_lines += report_data.blanks

    report.add_row(
        "Total",
        f"{total_files:,}",
        f"{total_lines:,}",
        f"{code_lines:,}",
        f"{comment_lines:,}",
        f"{blank_lines:,}",
    )
    return report


import unittest
import os
from unittest.mock import patch, MagicMock
from click.testing import CliRunner
from pylocc.cli import pylocc

class TestCli(unittest.TestCase):

    @patch('pylocc.cli.load_language_config')
    @patch('pylocc.cli.Processor')
    def test_pylocc_single_file(self, mock_processor, mock_load_config):
        # Arrange
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('test.py', 'w') as f:
                f.write('print("hello world")')

            mock_config = MagicMock()
            mock_config.file_extensions = ['py']
            mock_load_config.return_value = [mock_config]

            mock_proc_instance = mock_processor.return_value
            mock_proc_instance.process.return_value = MagicMock(total=1, code=1, comments=0, blanks=0, file_type='python')

            # Act
            result = runner.invoke(pylocc, ['test.py'])

            # Assert
            self.assertEqual(result.exit_code, 0)
            self.assertIn('Total', result.output)

    @patch('pylocc.cli.load_language_config')
    @patch('pylocc.cli.Processor')
    def test_pylocc_directory(self, mock_processor, mock_load_config):
        # Arrange
        runner = CliRunner()
        with runner.isolated_filesystem():
            os.makedirs('test_dir')
            with open('test_dir/test.py', 'w') as f:
                f.write('print("hello world")')

            mock_config = MagicMock()
            mock_config.file_extensions = ['py']
            mock_load_config.return_value = [mock_config]

            mock_proc_instance = mock_processor.return_value
            mock_proc_instance.process.return_value = MagicMock(total=1, code=1, comments=0, blanks=0, file_type='python')

            # Act
            result = runner.invoke(pylocc, ['test_dir'])

            # Assert
            self.assertEqual(result.exit_code, 0)
            self.assertIn('Total', result.output)

    @patch('pylocc.cli.load_language_config')
    @patch('pylocc.cli.Processor')
    def test_pylocc_by_file(self, mock_processor, mock_load_config):
        # Arrange
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open('test.py', 'w') as f:
                f.write('print("hello world")')

            mock_config = MagicMock()
            mock_config.file_extensions = ['py']
            mock_load_config.return_value = [mock_config]

            mock_proc_instance = mock_processor.return_value
            mock_proc_instance.process.return_value = MagicMock(total=1, code=1, comments=0, blanks=0, file_type='python')

            # Act
            result = runner.invoke(pylocc, ['--by-file', 'test.py'])

            # Assert
            self.assertEqual(result.exit_code, 0)
            self.assertIn('Provider', result.output)

if __name__ == '__main__':
    import os
    unittest.main()



from unittest import TestCase

from pylocc.processor import ProcessorConfiguration, ProcessorConfigurationFactory

class TestProcessorConfigurationFactory(TestCase):

    def setUp(self):
        self.text_config = ProcessorConfiguration(
            file_type='txt',
            file_extensions=['txt'],
            line_comment=["//"],
            multiline_comment=[("/*", "*/")]
        )
        self.sql_config = ProcessorConfiguration(
            file_type='sql',
            file_extensions=['sql'],
            line_comment=["--"],
            multiline_comment=[]
        )
        self.factory = ProcessorConfigurationFactory([self.sql_config, self.text_config])

    def test_should_return_configuration_for_text_file(self):
        config = self.factory.get_configuration('txt')
        assert config is not None
        self.assertEqual(config.file_type, 'txt')

        config = self.factory.get_configuration('sql')
        assert config is not None
        self.assertEqual(config.file_type, 'sql')

